#include "binheap.h"

#include <stdio.h>
#include <stdlib.h>		// For exit()
#include <assert.h>
#include <malloc.h>
#include <limits.h>		// For INT_MIN

void bhInit(BinaryHeap *heap, unsigned int max_size)
{
	assert(heap != NULL);

	heap->data = (Type *)calloc(max_size + 1, sizeof(Type));
	if (heap->data == NULL) {
		fprintf(stderr, "Memory Allocation Error!\nExiting Program...\n\n");
		exit(1);
	}
	heap->max_size = max_size;
	heap->current_size = 0;
	heap->data[0] = INT_MIN;
}

void bhDestroy(BinaryHeap *heap)
{
	assert(heap != NULL);

	if (heap->data != NULL)
		free(heap->data);
	heap->data = NULL;
	heap->max_size = 0;
	heap->current_size = 0;
}

void bhInsert(BinaryHeap *heap, Type value)
{
	int i;

	assert(heap != NULL);

	if(bhIsFull(heap) == TRUE)
	{
		fprintf(stderr, "Binary Heap with maximum size %u is full!\n\tElement with value "TYPE_MODIFIER" was not be inserted\n", heap->max_size, value);
		return;
	}

    ++heap->current_size;
	// Propagete the parent nodes of the new leaf downwards if their value is larger than the value being inserted
	for( i = heap->current_size; heap->data[i / 2] > value; i /= 2 )
		heap->data[i] = heap->data[i / 2];
	// Insert the new value
	heap->data[i] = value;
}


Type bhDeleteMin(BinaryHeap *heap)
{
	unsigned int i, child;
	Type min, last;

	assert(heap != NULL);

	if (bhIsEmpty(heap) == TRUE) {
		fprintf(stderr, "Binary Heap is empty! Returning dummy value...\n");
		return heap->data[0];
	}

	min = heap->data[1];
	last = heap->data[heap->current_size];
	--heap->current_size;

	for( i = 1; i * 2 <= heap->current_size; i = child )
	{
		/* Find smaller child */
		child = i * 2;
		if ((child != heap->current_size) && (heap->data[child + 1] < heap->data[child]))
			child = child + 1;

		/* Percolate one level */
		if( last > heap->data[ child ] )
			heap->data[ i ] = heap->data[ child ];
		else
			break;
	}
	heap->data[i] = last;

	return min;
}


BOOL bhIsEmpty(BinaryHeap *heap)
{
	assert(heap != NULL);
	if (heap->current_size == 0)
		return TRUE;
	else
		return FALSE;
}

BOOL bhIsFull(BinaryHeap *heap)
{
	assert(heap != NULL);
	
	if (heap->current_size == heap->max_size)
		return TRUE;
	else
		return FALSE;

}

void bhPrint(BinaryHeap *heap)
{
	unsigned int i = 0;

	assert(heap != NULL);

	for (i = 0; i < heap->current_size; i++)
	{
		fprintf(stdout, TYPE_MODIFIER" ", heap->data[i]);
	}
	fprintf(stdout, "\n");

}