#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "binheap.h"
#include "quadheap.h"

#define MAX_HEAP_SIZE 4000000

int main(void)
{
	int i = 0;
	Type value = 0;
	BinaryHeap bHeap;
	QuadHeap qHeap;
	Type *initialData = NULL;
	clock_t start = 0;
	clock_t duration = 0;
	int size = 0;


	initialData = (Type *)malloc(sizeof(Type) * MAX_HEAP_SIZE);
	if (initialData == NULL) {
		fprintf(stderr, "Memory Allocation Error\n");
		exit(1);
	}

	srand((unsigned int)time(NULL));

	for (i = 0; i < MAX_HEAP_SIZE; i++) {
        initialData[i] = rand();
	}

	printf("size binHeap_insert binHeap_delete quadHeap_insert quadHeap_delete\n");
	for (size = MAX_HEAP_SIZE; size > 0; size -= 100000)
	{
		printf("%d: ", size);
		bhInit(&bHeap, size);

		start = clock();
		for (i = 0; i < size; i++) {
			bhInsert(&bHeap, initialData[i]);
		}
		duration = clock() - start;
		printf("%d ",duration);

		start = clock();
		for (i = 0; i < size; i++) {
			bhDeleteMin(&bHeap);
		}
		duration = clock() - start;
		printf("%d ",duration);

		bhDestroy(&bHeap);


		qhInit(&qHeap, size);

		start = clock();
		for (i = 0; i < size; i++) {
			qhInsert(&qHeap, initialData[i]);
		}
		duration = clock() - start;

		printf("%d ", duration);

		start = clock();
		for (i = 0; i < size; i++) {
			qhDeleteMin(&qHeap);
		}
		duration = clock() - start;
		printf("%d\n",duration);

		qhDestroy(&qHeap);

	}

	if (initialData != NULL)
		free(initialData);

	return 0;
}
