#include "quadheap.h"

#include <stdio.h>
#include <stdlib.h>		// For exit()
#include <assert.h>
#include <malloc.h>
#include <limits.h>		// For INT_MIN

void qhInit(QuadHeap *heap, unsigned int max_size)
{
	assert(heap != NULL);

	heap->data = (Type *)calloc(max_size + 1, sizeof(Type));
	if (heap->data == NULL) {
		fprintf(stderr, "Memory Allocation Error!\nExiting Program...\n\n");
		exit(1);
	}
	heap->max_size = max_size;
	heap->current_size = 0;
	heap->data[0] = INT_MIN;
}

void qhDestroy(QuadHeap *heap)
{
	assert(heap != NULL);

	if (heap->data != NULL)
		free(heap->data);
	heap->data = NULL;
	heap->max_size = 0;
	heap->current_size = 0;
}

void qhInsert(QuadHeap *heap, Type value)
{
	int i;

	assert(heap != NULL);

	if(qhIsFull(heap))
	{
		fprintf(stderr, "Quad Heap with maximum size %u is full!\n\tElement with value "TYPE_MODIFIER" was not inserted\n", heap->max_size, value);
		return;
	}

    ++heap->current_size;
	// Propagete the parent nodes of the new leaf downwards if their value is larger than the value being inserted
	for( i = heap->current_size; heap->data[(i + 2) / 4] > value; i = (i + 2) / 4 )
		heap->data[i] = heap->data[(i + 2)/ 4];
	// Insert the new value
	heap->data[i] = value;
}


Type qhDeleteMin(QuadHeap *heap)
{
	unsigned int i, j, child, min_child;
	Type min, last;

	assert(heap != NULL);

	if (qhIsEmpty(heap) == TRUE) {
		fprintf(stderr, "Quad Heap is empty! Returning dummy value...\n");
		return heap->data[0];
	}

	min = heap->data[1];
	last = heap->data[heap->current_size];
	--heap->current_size;

	for( i = 1; (4 * i - 2) <= heap->current_size; i = min_child ) {
		/* Find smallest child */
		child = 4 * i - 2;
		min_child = child;
		for (j = child + 1; (j < (child + 4)) && (j < heap->current_size); j++) {
			if (heap->data[j] < heap->data[min_child])
				min_child = j;
		}

		/* Percolate one level */
		if( last > heap->data[min_child] )
			heap->data[ i ] = heap->data[ min_child ];
		else
			break;
	}
	heap->data[i] = last;

	return min;
}


BOOL qhIsEmpty(QuadHeap *heap)
{
	assert(heap != NULL);
	if (heap->current_size == 0)
		return TRUE;
	else
		return FALSE;
}

BOOL qhIsFull(QuadHeap *heap)
{
	assert(heap != NULL);
	
	if (heap->current_size == heap->max_size)
		return TRUE;
	else
		return FALSE;

}

void qhPrint(QuadHeap *heap)
{
	unsigned int i = 0;

	assert(heap != NULL);

	for (i = 0; i < heap->current_size; i++)
	{
		fprintf(stdout, TYPE_MODIFIER" ", heap->data[i]);
	}
	fprintf(stdout, "\n");

}